@@ -273,7 +273,7 @@ on_about1_activate (GtkButton *button, TotemEmbedded *emb)
 
 	emb->about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
 			"name", _("Totem Mozilla Plugin"),
-			"version", PACKAGE_VERSION,
+			"version", VERSION,
 			"copyright", _("Copyright \xc2\xa9 2002-2005 Bastien Nocera"),
 			"comments", description,
 			"authors", authors,
@@ -582,10 +582,6 @@ int main (int argc, char **argv)
 	gchar *svcname;
 	GError *e = NULL;
 
-	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
-
 	if (XInitThreads () == 0)
 	{
 		gtk_init (&argc, &argv);
