@@ -35,7 +35,6 @@
 
 #include "totem-uri.h"
 #include "totem-interface.h"
-#include "totem-pl-parser.h"
 #include "video-utils.h"
 #include "debug.h"
 
@@ -52,6 +51,12 @@ typedef struct {
 	PlaylistCallback func;
 } PlaylistTypes;
 
+typedef struct {
+	const char *name;
+	const char *suffix;
+	TotemPlParserType type;
+} PlaylistSaveType;
+
 struct TotemPlaylistPrivate
 {
 	GladeXML *xml;
@@ -112,6 +117,13 @@ enum {
 	NUM_COLS
 };
 
+static PlaylistSaveType save_types [] = {
+	{".PLS", ".pls", TOTEM_PL_PARSER_PLS},
+	{".M3U", ".m3u", TOTEM_PL_PARSER_M3U},
+	{".M3U (DOS)", ".m3u", TOTEM_PL_PARSER_M3U_DOS},
+	{".XSPF", ".xspf", TOTEM_PL_PARSER_XSPF},
+};
+
 static int totem_playlist_table_signals[LAST_SIGNAL] = { 0 };
 
 static const GtkTargetEntry target_table[] = {
@@ -214,8 +226,8 @@ totem_playlist_mrl_to_title (const gchar *mrl)
 
 	filename = g_path_get_basename (mrl);
 	unescaped = gnome_vfs_unescape_string_for_display (filename);
-	g_free (filename);
 
+	g_free (filename);
 	filename_for_display = g_filename_to_utf8 (unescaped,
 			-1,             /* length */
 			NULL,           /* bytes_read */
@@ -226,12 +238,8 @@ totem_playlist_mrl_to_title (const gchar *mrl)
 	{
 		filename_for_display = g_locale_to_utf8 (unescaped,
 				-1, NULL, NULL, NULL);
-		if (filename_for_display == NULL) {
-			filename_for_display = g_filename_display_name
-				(unescaped);
-		}
-		g_free (unescaped);
-		return filename_for_display;
+		if (filename_for_display == NULL)
+			return unescaped;
 	}
 
 	g_free (unescaped);
@@ -319,6 +327,12 @@ totem_playlist_save_get_iter_func (GtkTreeModel *model,
 
 void
 totem_playlist_save_current_playlist (TotemPlaylist *playlist, const char *output)
+{
+	totem_playlist_save_current_playlist_ext (playlist, output, TOTEM_PL_PARSER_PLS);
+}
+
+void
+totem_playlist_save_current_playlist_ext (TotemPlaylist *playlist, const char *output, TotemPlParserType type)
 {
 	GError *error = NULL;
 	gboolean retval;
@@ -326,7 +340,7 @@ totem_playlist_save_current_playlist (TotemPlaylist *playlist, const char *outpu
 	retval = totem_pl_parser_write (playlist->_priv->parser,
 			playlist->_priv->model,
                         totem_playlist_save_get_iter_func,
-			output, TOTEM_PL_PARSER_PLS, NULL, &error);
+			output, type, NULL, &error);
 
 	if (retval == FALSE)
 	{
@@ -807,10 +821,59 @@ totem_playlist_remove_files (GtkWidget *widget, TotemPlaylist *playlist)
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (playlist->_priv->treeview));
 }
 
+static void
+totem_playlist_save_playlist (TotemPlaylist *playlist, char *filename, gint active_format)
+{
+	PlaylistSaveType *cur = NULL;
+	guint i;
+
+	if (active_format > 0)
+		totem_playlist_save_current_playlist_ext (playlist, filename,
+							save_types[active_format - 1].type);
+	else {
+		for (i = 0; i < G_N_ELEMENTS(save_types); i++) {
+			if(g_str_has_suffix (filename, save_types[i].suffix)) {
+				cur = &save_types[i];
+				break;
+			}
+		}
+		if (cur == NULL)
+			totem_playlist_error (_("Could not save the playlist"), _("Unknown file extension."), playlist);
+		else
+			totem_playlist_save_current_playlist_ext (playlist, filename, cur->type);
+	}
+}
+
+static GtkWidget *
+totem_playlist_save_add_format_combo_box (GtkFileChooser *fc)
+{
+	GtkWidget *hbox, *label, *combo_box;
+	guint i;
+
+	hbox = gtk_hbox_new (FALSE, 4);
+	label = gtk_label_new (_("Select playlist format:"));
+	gtk_widget_show (label);
+
+	combo_box = gtk_combo_box_new_text ();
+	gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), _("By extension"));
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 0);
+	for (i = 0; i < G_N_ELEMENTS(save_types); i++) {
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), save_types[i].name);
+	}
+	gtk_widget_show (combo_box);
+
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), combo_box, TRUE, TRUE, 0);
+	gtk_widget_show (hbox);
+	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (fc), hbox);
+
+	return combo_box;
+}
+
 static void
 totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 {
-	GtkWidget *fs;
+	GtkWidget *fs, *combo_box;
 	int response;
 
 	fs = gtk_file_chooser_dialog_new (_("Save Playlist"),
@@ -821,6 +884,7 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 			NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
+	combo_box = totem_playlist_save_add_format_combo_box (GTK_FILE_CHOOSER (fs));
 
 	if (playlist->_priv->save_path != NULL)
 	{
@@ -838,8 +902,10 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 		char *filename;
 		GnomeVFSResult res;
 		GnomeVFSFileInfo info;
+		gint active_format;
 
 		filename = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (fs));
+		active_format = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
 
 		gtk_widget_destroy (fs);
 
@@ -878,8 +944,7 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 				return;
 			}
 		}
-
-		totem_playlist_save_current_playlist (playlist, filename);
+		totem_playlist_save_playlist (playlist, filename, active_format);
 		g_free (filename);
 	} else {
 		gtk_widget_destroy (fs);
