@@ -42,8 +42,6 @@
 
 #include "debug.h"
 
-#define PROPRIETARY_PLUGINS ".gnome2"G_DIR_SEPARATOR_S"totem-addons"
-
 static void
 totem_action_info (char *reason, Totem *totem)
 {
@@ -414,7 +412,7 @@ totem_setup_preferences (Totem *totem)
 	GtkWidget *item, *menu;
 	gboolean show_visuals, auto_resize, is_local, deinterlace;
 	int connection_speed, i;
-	char *path, *visual, *font;
+	char *visual, *font;
 	GList *list, *l;
 	BaconVideoWidgetAudioOutType audio_out;
 
@@ -438,8 +436,22 @@ totem_setup_preferences (Totem *totem)
 			disable_save_to_disk_changed_cb,
 			totem, NULL, NULL);
 
-	totem->prefs = glade_xml_get_widget (totem->xml,
-			"totem_preferences_window");
+	/* Work-around glade dialogue not parenting properly for
+	 * On top windows */
+	item = glade_xml_get_widget (totem->xml, "tpw_notebook");
+	totem->prefs = gtk_dialog_new_with_buttons ("Preferences",
+			GTK_WINDOW (totem->win),
+			GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_STOCK_CLOSE,
+			GTK_RESPONSE_ACCEPT,
+			NULL);
+	gtk_dialog_set_has_separator (GTK_DIALOG (totem->prefs), FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (totem->prefs), 5);
+	gtk_box_set_spacing (GTK_BOX(GTK_DIALOG(totem->prefs)->vbox), 2);
+	gtk_widget_reparent (item, GTK_DIALOG (totem->prefs)->vbox);
+	gtk_widget_show_all (GTK_DIALOG (totem->prefs)->vbox);
+	item = glade_xml_get_widget (totem->xml, "totem_preferences_window");
+	gtk_widget_destroy (item);
 
 	g_signal_connect (G_OBJECT (totem->prefs), "response",
 			G_CALLBACK (gtk_widget_hide), NULL);
@@ -463,14 +475,6 @@ totem_setup_preferences (Totem *totem)
 	g_signal_connect (item, "changed",
 			G_CALLBACK (connection_combobox_changed), totem);
 
-	/* Proprietary plugins */
-	path = g_build_path (G_DIR_SEPARATOR_S,
-			g_get_home_dir (), PROPRIETARY_PLUGINS, NULL);
-	if (g_file_test (path, G_FILE_TEST_IS_DIR) == FALSE)
-		mkdir (path, 0775);
-	bacon_video_widget_set_proprietary_plugins_path (totem->bvw, path);
-	g_free (path);
-
 	/* Enable visuals */
 	item = glade_xml_get_widget (totem->xml, "tpw_visuals_checkbutton");
 	show_visuals = gconf_client_get_bool (totem->gc,
@@ -631,9 +635,6 @@ totem_preferences_tvout_setup (Totem *totem)
 	case TV_OUT_NVTV_NTSC:
 		name = "tpw_nvtvntscmode_radio_button";
 		break;
-	case TV_OUT_DXR3:
-		name = "tpw_dxr3tvout_radio_button";
-		break;
 	default:
 		g_assert_not_reached ();
 		name = NULL;
@@ -665,14 +666,6 @@ totem_preferences_tvout_setup (Totem *totem)
                 GINT_TO_POINTER (TV_OUT_NVTV_NTSC));
 	gtk_widget_set_sensitive(item, 
 			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_NVTV_NTSC));
-	
-	item = glade_xml_get_widget (totem->xml, "tpw_dxr3tvout_radio_button");
-	g_signal_connect (G_OBJECT (item), "toggled",
-			G_CALLBACK (on_tvout_toggled), totem);
-	g_object_set_data (G_OBJECT (item), "tvout_type",
-			GINT_TO_POINTER (TV_OUT_DXR3));
-	gtk_widget_set_sensitive(item, 
-			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_DXR3));
 }
 
 void
