@@ -165,7 +165,8 @@ totem_action_exit (Totem *totem)
 {
 	GdkDisplay *display = NULL;
 
-	popup_timeout_remove (totem);
+	if (totem != NULL)
+		popup_timeout_remove (totem);
 
 	if (gtk_main_level () > 0)
 		gtk_main_quit ();
@@ -173,14 +174,15 @@ totem_action_exit (Totem *totem)
 	if (totem == NULL)
 		exit (0);
 
-	if (totem->playlist != NULL)
-		gtk_widget_hide (GTK_WIDGET (totem->playlist));
-
 	if (totem->win != NULL) {
 		gtk_widget_hide (totem->win);
 		display = gtk_widget_get_display (totem->win);
 	}
 
+	if (totem->prefs != NULL) {
+		gtk_widget_hide (totem->prefs);
+	}
+
 #ifndef HAVE_GTK_ONLY
 	totem_gromit_clear (TRUE);
 #endif /* !HAVE_GTK_ONLY */
@@ -206,8 +208,6 @@ totem_action_exit (Totem *totem)
 	if (totem->gc)
 		g_object_unref (G_OBJECT (totem->gc));
 
-	if (totem->playlist)
-		gtk_widget_destroy (GTK_WIDGET (totem->playlist));
 	if (totem->win)
 		gtk_widget_destroy (GTK_WIDGET (totem->win));
 
@@ -1199,13 +1199,18 @@ on_next_button_clicked (GtkButton *button, Totem *totem)
 static void
 on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
 {
-	g_message ("on_got_redirect %s", mrl);
-	//FIXME we need to check for relative paths here
+	gchar *old_mrl, *new_mrl;
+
+	old_mrl = totem_playlist_get_current_mrl (TOTEM_PLAYLIST (totem->playlist));
+	new_mrl = totem_resolve_relative_link (old_mrl, mrl);
+	g_free (old_mrl);
+
 	bacon_video_widget_close (totem->bvw);
 	totem_gdk_window_set_waiting_cursor (totem->win->window);
-	bacon_video_widget_open (totem->bvw, mrl, NULL);
+	bacon_video_widget_open (totem->bvw, new_mrl, NULL);
 	gdk_window_set_cursor (totem->win->window, NULL);
 	bacon_video_widget_play (bvw, NULL);
+	g_free (new_mrl);
 }
 
 /* This is only called when we are playing a DVD */
@@ -1827,6 +1832,7 @@ show_controls (Totem *totem, gboolean was_fullscreen)
 			width =	widget->allocation.width;
 		}
 
+		gtk_widget_set_sensitive (menubar, TRUE);
 		gtk_widget_show (menubar);
 		gtk_widget_show (controlbar);
 		gtk_widget_show (statusbar);
@@ -1851,7 +1857,10 @@ show_controls (Totem *totem, gboolean was_fullscreen)
 			height = widget->allocation.height;
 		}
 
+		/* Hide and make the menubar unsensitive */
+		gtk_widget_set_sensitive (menubar, FALSE);
 		gtk_widget_hide (menubar);
+
 		gtk_widget_hide (controlbar);
 		gtk_widget_hide (statusbar);
 		gtk_widget_hide (totem->sidebar);
@@ -1939,6 +1948,7 @@ on_about1_activate (GtkButton *button, Totem *totem)
 		"Chee Bin Hoh <cbhoh@gnome.org>",
 		NULL
 	};
+	const char *frontend_type;
 	char *backend_version, *description;
 	char *filename;
 
@@ -1953,9 +1963,17 @@ on_about1_activate (GtkButton *button, Totem *totem)
 	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
 	g_free (filename);
 
+#ifdef HAVE_GTK_ONLY
+	frontend_type = N_("GTK+");
+#else
+	frontend_type = N_("GNOME");
+#endif
+
 	backend_version = bacon_video_widget_get_backend_name (totem->bvw);
-	description = g_strdup_printf (_("Movie Player using %s"),
-				backend_version);
+	/* This lists the back-end and front-end types and versions, such as
+	 * Movie Player using GStreamer 0.10.1 and GNOME */
+	description = g_strdup_printf (_("Movie Player using %s and %s"),
+				backend_version, _(frontend_type));
 
 	totem->about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
 			"name", _("Totem"),
@@ -1993,6 +2011,9 @@ on_take_screenshot1_activate (GtkButton *button, Totem *totem)
 	char *filename;
 	GError *err = NULL;
 
+	if (bacon_video_widget_get_logo_mode (totem->bvw) != FALSE)
+		return;
+
 	if (bacon_video_widget_can_get_frames (totem->bvw, &err) == FALSE)
 	{
 		if (err == NULL)
@@ -2143,8 +2164,7 @@ on_skip_to1_activate (GtkButton *button, Totem *totem)
 		return;
 	}
 
-	filename = g_build_filename (DATADIR,
-			"totem", "skip_to.glade", NULL);
+	filename = totem_interface_get_full_path ("skip_to.glade");
 	totem->skipto = TOTEM_SKIPTO (totem_skipto_new (filename));
 	g_free (filename);
 
@@ -2685,7 +2705,11 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		break;
 	case GDK_s:
 	case GDK_S:
-		on_skip_to1_activate (NULL, totem);
+		if (event->state & GDK_CONTROL_MASK) {
+			on_take_screenshot1_activate (NULL, totem);
+		} else {
+			on_skip_to1_activate (NULL, totem);
+		}
 		break;
 	case GDK_t:
 	case GDK_T:
@@ -2737,7 +2761,7 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		}
 		break;
 	case GDK_space:
-		if (totem_is_fullscreen (totem) != FALSE)
+		if (totem_is_fullscreen (totem) != FALSE || gtk_widget_is_focus (GTK_WIDGET (totem->bvw)) != FALSE)
 			totem_action_play_pause (totem);
 		else
 			retval = FALSE;
